/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CreateVolumeDialog.h"
#include "QDebug.h"
#include "QConfig.h"
#include "DialogHelper.h"
#include "VolumeConfig.h"
#include "Util.h"
#include "MsgDialog.h"

#include "Pline/FSysUtil.h"

#include <glibmm/i18n.h>
#include <assert.h>

//==============================================================================

CreateVolumeDialog::CreateVolumeDialog(
		BaseObjectType* cobject,
		const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Create Encrypted Volume"));
	
	gpoDialogHelper = 0;
	gpoVolumeConfig = 0;
	gpvecVolumeConfigLabel = 0;
	
	UtilGui::mapCreateVolumeDlg(refGlade, gtCreateVolumeDlg);	
	
	// tCreatePlainDiskImageDlgItem
	gtCreateVolumeDlg.poCreateButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreateVolumeDialog::onCreate));

	gtCreateVolumeDlg.poCancelButton->signal_clicked().connect(
		sigc::mem_fun(*this, &CreateVolumeDialog::onCancel));
	
	gtCreateVolumeDlg.poSourceFileChooser->signal_selection_changed().connect(
		sigc::mem_fun(*this, &CreateVolumeDialog::onSource));
			
	Gtk::ComboBoxText* pocmbFSys = manage(new Gtk::ComboBoxText);
			
	gtCreateVolumeDlg.poFSysVBox->pack_start(*pocmbFSys, false, false);
	gtCreateVolumeDlg.poFSysComboBox = pocmbFSys;
	
	Gtk::ComboBoxText* pocmbPlaceAfter = manage(new Gtk::ComboBoxText);
			
	gtCreateVolumeDlg.poPlaceAfterVBox->pack_start(*pocmbPlaceAfter, false, false);
	gtCreateVolumeDlg.poPlaceAfterComboBox = pocmbPlaceAfter;
		
	gtCreateVolumeDlg.poUsePasswordRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &CreateVolumeDialog::onUsePassword));
	
	gtCreateVolumeDlg.poUseKeyFileRadioButton->signal_toggled().connect(
		sigc::mem_fun(*this, &CreateVolumeDialog::onUseKeyFile));
		
	Gtk::ComboBoxText* pocmbCipher = manage(new Gtk::ComboBoxText);
			
	gtCreateVolumeDlg.poCipherVBox->pack_start(*pocmbCipher, false, false);
	gtCreateVolumeDlg.poCipherComboBox = pocmbCipher;
	
	Gtk::ComboBoxText* pocmbCBC = manage(new Gtk::ComboBoxText);
			
	gtCreateVolumeDlg.poCBCVBox->pack_start(*pocmbCBC, false, false);
	gtCreateVolumeDlg.poCBCComboBox = pocmbCBC;
	
	Gtk::ComboBoxText* pocmbKeySize = manage(new Gtk::ComboBoxText);
			
	gtCreateVolumeDlg.poKeySizeVBox->pack_start(*pocmbKeySize, false, false);
	gtCreateVolumeDlg.poKeySizeComboBox = pocmbKeySize;
	
	show_all_children();
}

//==============================================================================

CreateVolumeDialog::~CreateVolumeDialog()
{
	if (gpoDialogHelper)
		delete gpoDialogHelper;
}

//==============================================================================

void CreateVolumeDialog::buildFSys()
{
	unsigned int xui;

	gvecFSys.clear();
	
	gtCreateVolumeDlg.poFSysComboBox->clear_items();
	
	Pline::FSysUtil::getMkFSysList(&gvecFSys);

	for (xui=0; xui<gvecFSys.size(); ++xui)
		gtCreateVolumeDlg.poFSysComboBox->append_text(gvecFSys[xui]);
}

//==============================================================================

void CreateVolumeDialog::onCancel()
{
	QDialog::response(Gtk::RESPONSE_CANCEL);
}

//==============================================================================

void CreateVolumeDialog::onSource()
{
	gpoDialogHelper->onSource();
}

//==============================================================================

void CreateVolumeDialog::onCreate()
{
	updateData(false);
	
	if (!validate())
		return;
		
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void CreateVolumeDialog::onUseKeyFile()
{
	gpoDialogHelper->onUseKeyFile();
}

//==============================================================================

void CreateVolumeDialog::onUsePassword()
{
	gpoDialogHelper->onUsePassword();
}

//==============================================================================

void CreateVolumeDialog::setData(   VolumeConfig *poVolumeConfig,
									std::vector<Glib::ustring> *pvecVolumeConfigLabel,
									Glib::ustring *pstCipher,
									Glib::ustring *pstCBC,
									Glib::ustring *pstKeySize)
{
	gpoVolumeConfig = poVolumeConfig;
	gpvecVolumeConfigLabel = pvecVolumeConfigLabel;
	
	buildFSys();
	
	//
	// Dialog Helper
	//
	
	if (gpoDialogHelper)
		delete gpoDialogHelper;

	gpoDialogHelper = new DialogHelper(this, gpoVolumeConfig);
	
	// mount
	gpoDialogHelper->setSourceFileChooser(gtCreateVolumeDlg.poSourceFileChooser);
	gpoDialogHelper->setMkFSysComboBox(gtCreateVolumeDlg.poFSysComboBox);
	gpoDialogHelper->setFSysList(&gvecFSys);
	gpoDialogHelper->setPlaceAfterComboBox(gtCreateVolumeDlg.poPlaceAfterComboBox);
	gpoDialogHelper->setPlaceAfterList(gpvecVolumeConfigLabel);
	gpoDialogHelper->setVolumeLabelEntry(gtCreateVolumeDlg.poVolumeLabelEntry);
	gpoDialogHelper->setKeyFileFileChooser(gtCreateVolumeDlg.poKeyFileFileChooser);
	gpoDialogHelper->setUseKeyFileRadioButton(gtCreateVolumeDlg.poUseKeyFileRadioButton);
	gpoDialogHelper->setUsePasswordRadioButton(gtCreateVolumeDlg.poUsePasswordRadioButton);
	gpoDialogHelper->setEnableEncryption(true);
												
	gpoDialogHelper->setCipherComboBox(	gtCreateVolumeDlg.poCipherComboBox,
										pstCipher);
	gpoDialogHelper->setCBCComboBox(gtCreateVolumeDlg.poCBCComboBox,
									pstCBC);
	gpoDialogHelper->setKeySizeComboBox(gtCreateVolumeDlg.poKeySizeComboBox,
										pstKeySize);
	
	gpoDialogHelper->buildPlaceAfter();
	gpoDialogHelper->buildCiphers();
		
	updateData(true);
}
					
//==============================================================================

void CreateVolumeDialog::updateData(const bool bToScreen)
{
	assert(gpoVolumeConfig);
	
	gpoDialogHelper->updateCreateVolumeData(bToScreen);
}

//==============================================================================

bool CreateVolumeDialog::validate()
{
	Glib::ustring stErr;
	
	if (!gpoDialogHelper->validateMountData(true))
		return false;
		
	if (!gpoDialogHelper->validateCryptData())
		return false;
	
	// mount validation allows files and devices, make sure it's a device
	if (!Util::isDevice(
			gtCreateVolumeDlg.poSourceFileChooser->get_filename(),
			&stErr))
	{
		MsgDialog::tConfig tConfigItem;
		MsgDialog::initConfig(tConfigItem);
			
		tConfigItem.bAutoRun = true;
		tConfigItem.bWaitData = false;
		tConfigItem.poParent = this;
		tConfigItem.stMsg = stErr;
		
		MsgDialog oDlg(tConfigItem);
		
		return false;
	}
		
		
	return true;
}

//==============================================================================
