/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PermDialog.h"
#include "QDebug.h"
#include "Util.h"

#include <glibmm/i18n.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
	
static Glib::ustring zstPerm[] = {	
	Util::compose("%d", PermDialog::kPermOwner), 	_("Read only"), 		Util::compose("%d", S_IRUSR),
	Util::compose("%d", PermDialog::kPermOwner), 	_("Read & Write"), 		Util::compose("%d", S_IWUSR),
	Util::compose("%d", PermDialog::kPermGroup), 	_("No Access"), 		Util::compose("%d", 0),
	Util::compose("%d", PermDialog::kPermGroup), 	_("Read only"), 		Util::compose("%d", S_IRGRP),
	Util::compose("%d", PermDialog::kPermGroup), 	_("Read & Write"), 		Util::compose("%d", S_IWGRP),
	Util::compose("%d", PermDialog::kPermOther), 	_("No Access"), 		Util::compose("%d", 0),
	Util::compose("%d", PermDialog::kPermOther), 	_("Read only"), 		Util::compose("%d", S_IROTH),
	Util::compose("%d", PermDialog::kPermOther), 	_("Read & Write"), 		Util::compose("%d", S_IWOTH),
	"", 													"", 					"",
};

//==============================================================================

PermDialog::PermDialog( BaseObjectType* cobject,
						const Glib::RefPtr<Gtk::Builder>& refGlade)
: QDialog(cobject)
{
	set_modal(true);
	set_title(_("Ownership & Permissions"));
	
	gpoMountData = 0;
	
	UtilGui::mapPermDlg(refGlade, gtPermDlg);	
				
	gtPermDlg.poButton->signal_clicked().connect(
		sigc::mem_fun(*this, &PermDialog::onClose));
	
	gtPermDlg.poSetCheckButton->signal_toggled().connect(
		sigc::mem_fun(*this, &PermDialog::onSetPerm));
		
	Gtk::ComboBoxText* pocmbOwner = manage(new Gtk::ComboBoxText);
			
	gtPermDlg.poOwnerVBox->pack_start(*pocmbOwner, false, false);
	gtPermDlg.poOwnerComboBox = pocmbOwner;
		
	Gtk::ComboBoxText* pocmbGroup = manage(new Gtk::ComboBoxText);
			
	gtPermDlg.poGroupVBox->pack_start(*pocmbGroup, false, false);
	gtPermDlg.poGroupComboBox = pocmbGroup;
		
	Gtk::ComboBoxText* pocmbOwnerPerm = manage(new Gtk::ComboBoxText);
			
	gtPermDlg.poOwnerPermVBox->pack_start(*pocmbOwnerPerm, false, false);
	gtPermDlg.poOwnerPermComboBox = pocmbOwnerPerm;
	
	Gtk::ComboBoxText* pocmbGroupPerm = manage(new Gtk::ComboBoxText);
			
	gtPermDlg.poGroupPermVBox->pack_start(*pocmbGroupPerm, false, false);
	gtPermDlg.poGroupPermComboBox = pocmbGroupPerm;

	Gtk::ComboBoxText* pocmbOtherPerm = manage(new Gtk::ComboBoxText);
			
	gtPermDlg.poOtherPermVBox->pack_start(*pocmbOtherPerm, false, false);
	gtPermDlg.poOtherPermComboBox = pocmbOtherPerm;
		
	buildPerm();
	buildUsers();
	
	show_all_children();
}

//==============================================================================

PermDialog::~PermDialog()
{
}

//==============================================================================

void PermDialog::buildPerm()
{
	Glib::ustring st;
	long l;
	unsigned int cui = 0;
	unsigned int xui;
	tPerm tPerm;
	
	while (!(st = zstPerm[cui]).empty())
	{
		l = atol(st.c_str());
		tPerm.ePerm = (ePerm)l;

		st = zstPerm[cui+1];
		tPerm.stPerm = st;

		st = zstPerm[cui+2];
		l = atol(st.c_str());
		tPerm.tMode = (mode_t)l;

		gvecPerm.push_back(tPerm);
		
		cui+=3;
	}

	for (xui=0; xui<gvecPerm.size(); xui++)
	{
		Gtk::ComboBoxText *pocmb = 0;

		if (gvecPerm[xui].ePerm == kPermOwner)
			pocmb = gtPermDlg.poOwnerPermComboBox;
		else if (gvecPerm[xui].ePerm == kPermGroup)
			pocmb = gtPermDlg.poGroupPermComboBox;
		else if (gvecPerm[xui].ePerm == kPermOther)
			pocmb = gtPermDlg.poOtherPermComboBox;

		if (pocmb)
		{
			pocmb->append_text(gvecPerm[xui].stPerm);
			gvecPerm[xui].pocmb = pocmb;
		}
	}
}

//==============================================================================

void PermDialog::buildUsers()
{	
	if (!Util::getUsers(&gvecUserName, &gvecUserID))
		return;
	if (!Util::getGroups(&gvecGroupName, &gvecGroupID))
		return;
	
	// users
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < gvecUserName.size();
			++xi)
	{
		if (gvecUserID[xi] == 0 || gvecUserID[xi] >= 500)
			gtPermDlg.poOwnerComboBox->append_text(gvecUserName[xi]);
	}
	
	// groups
	for (   std::vector<Glib::ustring>::size_type xi = 0;
			xi < gvecGroupName.size();
			++xi)
	{
		if (gvecGroupID[xi] == 0 || gvecGroupID[xi] >= 500)
			gtPermDlg.poGroupComboBox->append_text(gvecGroupName[xi]);
	}
}

//==============================================================================

void PermDialog::enablePerm(const bool bEnable)
{	
	gtPermDlg.poSetCheckButton->set_active(bEnable);
	gtPermDlg.poOwnerComboBox->set_sensitive(bEnable);
	gtPermDlg.poGroupComboBox->set_sensitive(bEnable);
	gtPermDlg.poOwnerPermComboBox->set_sensitive(bEnable);
	gtPermDlg.poGroupPermComboBox->set_sensitive(bEnable);
	gtPermDlg.poOtherPermComboBox->set_sensitive(bEnable);
}

//==============================================================================

void PermDialog::onClose()
{
	updateData(false);
	
	QDialog::response(Gtk::RESPONSE_OK);
}

//==============================================================================

void PermDialog::onSetPerm()
{	
	enablePerm(gtPermDlg.poSetCheckButton->get_active());
}

//==============================================================================

void PermDialog::setData(Pline::MountData* poMountData)
{
	gpoMountData = poMountData;
		
	updateData(true);
}

//==============================================================================

void PermDialog::updateData(const bool bToScreen)
{
	assert(gpoMountData);
			
	if (bToScreen)
	{
		if (gpoMountData->getDirSetPerm())
		{
			enablePerm(true);
			updatePerm(gpoMountData->getDirPerm());
			updateUser( gpoMountData->getDirOwner(),
						gpoMountData->getDirGroup());
		}
		else
		{
			enablePerm(false);
			updatePerm();
			updateUser();
		}
	}
	else
	{
		if (gtPermDlg.poSetCheckButton->get_active())
		{
			unsigned long ulPerm = 0;
		
			for (unsigned int xui=0; xui<gvecPerm.size(); ++xui)
			{			
				if (gvecPerm[xui].pocmb->get_active_text() == gvecPerm[xui].stPerm)
				{
					if (gvecPerm[xui].tMode != 0)
						ulPerm |= (unsigned long)gvecPerm[xui].tMode;
				}
			}
		
			gpoMountData->setDirOwner(
				gtPermDlg.poOwnerComboBox->get_active_text());
			gpoMountData->setDirGroup(
				gtPermDlg.poGroupComboBox->get_active_text());
				
			gpoMountData->setDirPerm(ulPerm);
			gpoMountData->setDirSetPerm(true);
		}
		else
		{
			gpoMountData->setDirOwner("");
			gpoMountData->setDirGroup("");
			gpoMountData->setDirPerm(0);
			gpoMountData->setDirSetPerm(false);
		}
	}
}

//==============================================================================

void PermDialog::updatePerm(const mode_t tMode)
{
	unsigned int xui;
	struct stat tBuf;
	mode_t tTestMode = tMode;

	// Default to first
	for (xui=0; xui<gvecPerm.size(); ++xui)
		gvecPerm[xui].pocmb->set_active(0);
		
	if (!gpoMountData)
		return;
		
	if (tTestMode == 0)
	{
		if (stat(gpoMountData->getDir().c_str(), &tBuf) != 0)
			return;
		
		tTestMode = tBuf.st_mode;
	}
	
	// Match mask and set value
	for (xui=0; xui<gvecPerm.size(); ++xui)
	{
		if ((tTestMode & (gvecPerm[xui].tMode)))
		{
			gvecPerm[xui].pocmb->set_active_text(gvecPerm[xui].stPerm);
			break;
		}
	}
}

//==============================================================================

void PermDialog::updateUser(const Glib::ustring stUserName,
							const Glib::ustring stGroupName)
{
	unsigned int xui;
	struct stat tBuf;
	
	// Default to first
	gtPermDlg.poOwnerComboBox->set_active(0);
	gtPermDlg.poGroupComboBox->set_active(0);
	
	
	if (!stUserName.empty() && !stGroupName.empty())
	{
		gtPermDlg.poOwnerComboBox->set_active_text(stUserName);
		gtPermDlg.poGroupComboBox->set_active_text(stGroupName);
		
		return;
	}
	
	if (!gpoMountData)
		return;
				
	if (stat(gpoMountData->getDir().c_str(), &tBuf) != 0)
		return;

	// Select user
	for (xui=0; xui<gvecUserID.size(); ++xui)
	{
		if ((unsigned long)gvecUserID[xui] == (unsigned long)tBuf.st_uid)
		{
			gtPermDlg.poOwnerComboBox->set_active_text(gvecUserName[xui]);	
			break;
		}
	}

	// Select group
	for (xui=0; xui<gvecGroupID.size(); ++xui)
	{
		if ((unsigned long)gvecGroupID[xui] == (unsigned long)tBuf.st_gid)
		{
			gtPermDlg.poGroupComboBox->set_active_text(gvecGroupName[xui]);	
			break;
		}
	}
}

//==============================================================================
