/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COPYFILTER_H
#define COPYFILTER_H

#include "ProcessBaseFilter.h"
#include "CloneData.h"

#include <gtkmm.h>

namespace Pline 
{

class CopyFilter : public ProcessBaseFilter
{
public:
	CopyFilter();
	~CopyFilter();

	// ready to enter data
	void askData();

	// filter config
	inline const Glib::ustring getFilterName() const {return "Copy";}

	// data
	void setSrcData(CloneData *poData);
	void setLimit(const unsigned long long ullVolumeSize);
	// other data
	inline void setCurrentSpeed(float *pfCurrentSpeed) {gpfCurrentSpeed = pfCurrentSpeed;}
	inline void setETA(Glib::ustring *pstETA) {gpstETA = pstETA;}
	inline void setPctDone(int *piPctDone) {gpiPctDone = piPctDone;}
	
	
	// run mode
	typedef enum _RUNMODE_
	{
		kNoRunMode,
		kRunCopyFileStartNew,
		kRunCopyWithLimitStartNew,
		kRunTruncateWithLimitStartNew,
		kRunCopyEnlargeExpand,
	}
	eRunMode;

	inline void setRunMode(eRunMode eRunModeItem) {geRunMode = eRunModeItem;}
	
	// run
	bool run();
	
protected:

private:
	eRunMode geRunMode;
	eRunMode geRunModeLast;
	
	CloneData *gpoCloneData;
	float *gpfCurrentSpeed;
	int *gpiPctDone;
	Glib::ustring *gpstETA;
	
	off64_t gtStatSizeTotal;
	unsigned long long gullSizeByte;
	
	bool copy(	const Glib::ustring& stSrc,
				const Glib::ustring& stDest,
				const bool bEnlarge, 
				const bool bWithLimit);
	
	bool truncate(const Glib::ustring& stDest);
	
	void calculateETA();
	bool canOverwrite();
	bool isValidData(Glib::ustring *pstReason = 0);
};

} // Pline
#endif /* COPYFILTER_H */
