/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Filter.h"
#include "../QDebug.h"

namespace Pline
{

//==============================================================================

Filter::Filter()
{
	gpoSrcData = 0;
	gpoDestData = 0;
	
	gbCancelDataReq = false;
	gbExitDataReq = false;
}

//==============================================================================

Filter::~Filter()
{
}

//==============================================================================

void Filter::askData()
{
	bool bUnlock = true;
	
	if (!gmtxReqData.trylock()) // in one single thread this is true
		bUnlock = false;
		
	gbExitDataReq = true;
	
	releaseData();
	
	if (bUnlock)
		gmtxReqData.unlock();
}

//==============================================================================

void Filter::cancelDataReq()
{
	bool bUnlock = true;
	
	if (!gmtxReqData.trylock()) // in one single thread this is true
		bUnlock = false;

	gbExitDataReq = true;
	gbCancelDataReq = true;
	
	releaseData();

	if (bUnlock)
		gmtxReqData.unlock();
}

//==============================================================================

void Filter::releaseData()
{
	gcondReqData.signal();
	
	debug_message(DEBUG_PLINE, "Data has been released. Filter[%s]", getFilterName().c_str());
}

//==============================================================================

void Filter::requestData()
{
	debug_message(DEBUG_PLINE, "Request for data signaled. Filter[%s]", getFilterName().c_str());

	Glib::Mutex::Lock lock(gmtxReqData);

	// reset global data
	gbCancelDataReq = false;
	gbExitDataReq = false;
	
	m_signal_req_data();
}

//==============================================================================

void Filter::setSrcData(Data *poData)
{
	gpoSrcData = poData;
}

//==============================================================================


void Filter::setDestData(Data *poData)
{
	gpoDestData = poData;
}

//==============================================================================

void Filter::update()
{
	m_signal_update();
}

//==============================================================================
	
bool Filter::waitData()
{	
	Glib::Mutex::Lock lock(gmtxReqData);

	while (!gbExitDataReq)
		gcondReqData.wait(gmtxReqData);
	
	return (gbCancelDataReq) ? false : true;
}

//==============================================================================

} // Pline
