/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FILTER_H
#define FILTER_H

#include "Data.h"

#include <gtkmm.h>

namespace Pline
{

class Filter
{
public:
	Filter();
	virtual ~Filter();

	// ready to enter data
	virtual void askData();
	
	// filter config
	virtual const Glib::ustring getFilterName() const = 0;
	virtual const Glib::ustring getFilterType() {return "Base";}
	
	// run
	virtual bool run() = 0;
	
	// error
	inline const Glib::ustring& getLastError() {return gstLastError;}
	inline const bool getHasCancelledDataReq() {return gbCancelDataReq;}
	
	// data
	inline Data* getSrcData() {return gpoSrcData;}
	
	// signals
	inline sigc::signal<void>& signal_req_data() {return m_signal_req_data;}
	inline sigc::signal<void>& signal_update() {return m_signal_update;}
	//inline sigc::signal<void, int, const Glib::ustring&>& 
		//signal_run_error() {return m_signal_run_error;}
	
protected:
	Data *gpoSrcData;
	Data *gpoDestData;
	
	Glib::ustring gstLastError;
	
	void setDestData(Data *poData);
	void setSrcData(Data *poData);
	
	void cancelDataReq();
	void requestData();
	bool waitData();
	
	void update();
	
private:
	Glib::Mutex gmtxReqData;
	Glib::Cond  gcondReqData;
	bool		gbExitDataReq;
	bool		gbCancelDataReq;
	
	sigc::signal<void> m_signal_req_data;
	sigc::signal<void> m_signal_update;
	//sigc::signal<void, int, const Glib::ustring&> m_signal_run_error;
	
	void releaseData();
};

} // Pline	
#endif /* FILTER_H */
