/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MOUNTPLINE_H
#define MOUNTPLINE_H

#include "Pline.h"
#include "MountBaseFilter.h"

#include <gtkmm.h>

namespace Pline
{

class PermFilter;
class TrashFilter;

class MountPline : public Pline
{
public:
	MountPline( const eDataMode eDataModeItem,
				VolumeConfig *poVolumeConfig);
	virtual ~MountPline();
		
	// run mode
	typedef enum _RUNMODE_
	{
		kNoRunMode,
		kRunDiskUsage,
		kRunDumpStatus,
		kRunEject,
		kRunMount,
		kRunProbe,
	}
	eRunMode;
	
	inline eRunMode getLastRunMode() {return geLastRunMode;}
	inline void setRunMode(eRunMode eRunModeItem) {geRunMode = eRunModeItem;}
	
	bool run();
	
	// disk usage
	typedef struct _DISKUSAGE_
	{
		bool bValid;
		Glib::ustring stDiskSize;
		Glib::ustring stSpaceUsed;
		Glib::ustring stSpaceAvailable;
		int	iUsedSpacePct;
	}
	tDiskUsage;
	
	inline tDiskUsage getDiskUsage() {return gtDiskUsage;}
	
	// mounted volume status
	inline const bool isEncrypted() {return gbEncrypted;}
	inline const Glib::ustring& getFSys() {return gstFSys;}
	inline const Glib::ustring& getCipher() {return gstCipher;}
	
	// current volumeconfig
	inline VolumeConfig* getVolumeConfig() {return gpoVolumeConfig;}
	
	// dump status
	typedef struct _DUMPSTATUS_
	{
		Glib::ustring stFilterName;
		Glib::ustring stSrc;
		Glib::ustring stSrcType;
	}
	tDumpStatus;
	
	std::vector<tDumpStatus>& getDumpStatus();
	
	// utility
	bool isInUse(bool *pbMounted = 0);
	
	void updatePline(); // force pline update
	
protected:
	virtual bool onFilterPreRun(Filter *poFilter);
	virtual bool onFilterRun(Filter *poFilter, bool bHasRunWithErrors);
	
private:
	eRunMode	geRunMode;
	eRunMode	geLastRunMode;
	eRunMode	geCurrRunMode;
	bool		gbHasProbedFilters;
	bool 		gbProbeError;
	bool		gbDumpStatus;
	
	// Permssions
	PermFilter *gpoPermFilter;
	TrashFilter *gpoTrashFilter;
	
	// current config
	VolumeConfig *gpoVolumeConfig;
	
	// save last mount data
	Glib::ustring gstFSys;
	bool gbEncrypted;
	Glib::ustring gstCipher;
	
	void initMountStatus();
	
	// mount type
	typedef enum _MOUNTTYPE_
	{
		kNoMount,
		kMountSimple,
		kMountLoop,
		kMountCrypto,
		kMountLCrypto,
	}
	eMountType;
	
	static eMountType getMountType(VolumeConfig *poVolumeConfig);

	// disk usage
	tDiskUsage gtDiskUsage;
	
	bool calcDiskUsage();
	void initDiskUsage();
	
	// dump status
	void insertDumpItem(Filter* poFilter);
	std::vector<tDumpStatus> gvecDumpStatus;
	
	// create pline
	void createVolumeConfigPline(VolumeConfig *poVolumeConfig);
	
	// update pline
	void updateVolumeConfigPline(VolumeConfig *poVolumeConfig);
	
	// run
	bool dumpStatus();
	bool eject();
	bool mount();
	bool probe();
	
	// utility
	bool setFiltersRunMode(MountBaseFilter::eRunMode eRunModeItem);
	
};

} // Pline	
#endif /* MOUNTPLINE_H */
