/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QDEBUG_H
#define QDEBUG_H

#include <gtkmm.h>

#include <stdio.h>
#include <glib.h>

#ifdef QDEBUG
#include <iostream>
using std::cout;
using std::cerr;
#endif

enum QDEBUG_MESSAGE_FLAG
{
	NO_DEBUG		= 0,
	DEBUG_INFO		= 1 << 0,
	DEBUG_APP	    = 1 << 1,
	DEBUG_PLINE		= 1 << 2,

	DEBUG_ALL		= 1 << 20
};

void __debug_init(int flags);
void __debug(   int flag,
				const gchar* file, 
				gint line, 
				const gchar* fonction);
void __debug_message(   int flag, 
						const gchar* file, 
						gint line, 
						const gchar* fonction, 
						const char *string, ...);

bool debug_check_flags(int flags);

#ifdef QDEBUG
	#define debug_init(flags) __se_debug_init(flags);
	#define debug(flag) if(debug_check_flags(flag)) { __debug(flag, __FILE__, __LINE__, __FUNCTION__); }
	#define debug_message(flag, ...) if(debug_check_flags(flag)) { __debug_message(flag, __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__); }
#else
	#define debug_init(flags)
	#define debug(flag)
	#define debug_message(flag, ...)
#endif // QDEBUG
	
#endif /* QDEBUG_H */
