/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#ifndef VOLCELLRENDERER_H
#define VOLCELLRENDERER_H

#include <gtkmm.h>

#include "VolumeTreeView.h"

class VolumeCellRenderer : public Gtk::CellRenderer
{
public:
	VolumeCellRenderer();
	virtual ~VolumeCellRenderer();

	// buttons state
	typedef enum
	{
		kBtnHidden = 0,
		kBtnNormal = 1,
		kBtnDisabled = 2,
	}
	eBtnState;
	
	// cursor over controls
	typedef enum _LAYOUT_CTRLTYPE_
	{
		kBtnCancel,
		kBtnEject,
		kBtnMount,
	}
	eLayoutCtrlType;
	
	typedef struct _LAYOUT_CTRL_
	{
		int					iIndex;
		eBtnState			eBtnStateItem;
		eLayoutCtrlType		eLayoutCtrlTypeItem;
	}
	tLayoutCtrl;
	
	bool isControl(	const int ix, 
					const int iy,
					const int iRowIndexStart, // first visible item
					const int iRowIndexEnd, // last visible item
					tLayoutCtrl *ptLayoutCtrl); // all visible rows must be drawn first

	// row
	int calcRowHeight(  const Glib::ustring& stDrawInfo,
						const Glib::ustring& stDrawLabel,
						const bool bDrawProgress);

	int createNewRowIndex();
	bool removeRowIndex(const int iIndex);
	void setIndexOrder(const int iIndex, const int iPos);
	
	// properties
	Glib::PropertyProxy<int>				property_index();
	Glib::PropertyProxy<int>				property_pos();
	Glib::PropertyProxy<int>				property_btn_cancel();
	Glib::PropertyProxy<int>				property_btn_eject();
	Glib::PropertyProxy<int>				property_btn_mount();
	Glib::PropertyProxy<Glib::ustring>		property_draw_info();
	Glib::PropertyProxy<Glib::ustring>		property_draw_label();
	Glib::PropertyProxy<bool>				property_draw_progress_full();
	Glib::PropertyProxy<int>				property_draw_progress_pct_gauge();
	Glib::PropertyProxy<int>				property_draw_progress_pct_used();
	
	// signals
	inline sigc::signal<void, int, int, int>& signal_activate() {return m_signal_activate;}
	
protected:
	virtual bool activate_vfunc(GdkEvent* event, 
								Gtk::Widget& widget, 
								const Glib::ustring& path, 
								const Gdk::Rectangle& background_area, 
								const Gdk::Rectangle& cell_area, 
								Gtk::CellRendererState flags);
	
	virtual void render_vfunc(  const Glib::RefPtr<Gdk::Drawable>& window, 
								Gtk::Widget& widget, 
								const Gdk::Rectangle& background_area, 
								const Gdk::Rectangle& cell_area, 
								const Gdk::Rectangle& expose_area, 
								Gtk::CellRendererState flags);
	
private:
	// cache
	double gdScreenResolution;
	
	// properties
	Glib::Property<int>				m_property_index;
	Glib::Property<int>				m_property_pos;
	Glib::Property<int>				m_property_btn_cancel;
	Glib::Property<int>				m_property_btn_eject;
	Glib::Property<int>				m_property_btn_mount;
	Glib::Property<Glib::ustring>	m_property_draw_info;
	Glib::Property<Glib::ustring>	m_property_draw_label;
	Glib::Property<bool>			m_property_draw_progress_full;
	Glib::Property<int>				m_property_draw_progress_pct_gauge;
	Glib::Property<int>				m_property_draw_progress_pct_used;
	
	// signals
	sigc::signal<void, int, int, int>  m_signal_activate;
	
	// layout
	typedef struct _LAYOUT_
	{
		int iIndex; // random id number
				
		// buttons
		eBtnState	   eBtnCancelState;
		Gdk::Rectangle oBtnCancelRect;
		
		eBtnState	   eBtnEjectState;
		Gdk::Rectangle oBtnEjectRect;
		
		eBtnState	   eBtnMountState;
		Gdk::Rectangle oBtnMountRect;
		
		bool operator==(const struct _LAYOUT_ &tLayoutItem) const
		{
			return tLayoutItem.iIndex == iIndex;
		}
	}
	tLayout;
	std::list<tLayout> glstLayout;
	
	void updateLayout(tLayout *ptLayout);
	
	// progress
	void drawProgressBkg(   Cairo::RefPtr<Cairo::Context>& cr, 
							const Gdk::Rectangle& rectangle,
							const int iHeight,
							Gdk::Rectangle *poInnerRect);
	void drawProgressFull(  Cairo::RefPtr<Cairo::Context>& cr, 
							const Gdk::Rectangle& rectangle,
							const int iHeight);
	void drawProgressGauge( Cairo::RefPtr<Cairo::Context>& cr, 
							const Gdk::Rectangle& rectangle,
							const int iPct);
	void drawProgressUsedSpace( Cairo::RefPtr<Cairo::Context>& cr, 
								const Gdk::Rectangle& rectangle,
								const int iPctUsed);
	
	bool isProgressVisible();
					
	// utility
	inline bool isCursorInRect( const int ix, 
								const int iy, 
								const Gdk::Rectangle& rectangle)
		{
			if (!rectangle.has_zero_area()
				&& ix >= rectangle.get_x()
				&& ix <= rectangle.get_x() + rectangle.get_width()
				&& iy >= rectangle.get_y()
				&& iy <= rectangle.get_y() + rectangle.get_height())
			{
				return true;
			}
	
			return false;
		} // isCursorInRect
};
#endif /* VOLCELLRENDERER_H */
