/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "VolumeConfig.h"
#include "QDebug.h"
#include "Util.h"

//==============================================================================

VolumeConfig::VolumeConfig()
{
	gpoData = 0;
	gpoCloneData = 0;
	gpoCryptData = 0;
	gpoMountData = 0;
	
	gbAutomount = false;
}

//==============================================================================

VolumeConfig::~VolumeConfig()
{
	if (gpoData)
		delete gpoData;
		
	if (gpoCloneData)
		delete gpoCloneData;
	
	if (gpoCryptData)
		delete gpoCryptData;
	
	if (gpoMountData)
		delete gpoMountData;
}

//==============================================================================

VolumeConfig* VolumeConfig::operator=(const VolumeConfig &oVolumeConfig)
{
	// data
	gpoData = new Pline::Data;
	*gpoData = *oVolumeConfig.gpoData;
	
	// clone data
	if (oVolumeConfig.gpoCloneData)
	{
		if (gpoCloneData)
			delete gpoCloneData;
			
		gpoCloneData = new Pline::CloneData(*gpoData);
		*gpoCloneData = *oVolumeConfig.gpoCloneData;
	}
	else
	{
		if (gpoCloneData)
			removeCloneData();
	}
	
	// crypt data
	if (oVolumeConfig.gpoCryptData)
	{
		if (gpoCryptData)
			delete gpoCryptData;
			
		gpoCryptData = new Pline::CryptData(*gpoData);
		*gpoCryptData = *oVolumeConfig.gpoCryptData;
	}
	else
	{
		if (gpoCryptData)
			removeCryptData();
	}
	
	// mount data
	if (oVolumeConfig.gpoMountData)
	{
		if (gpoMountData)
			delete gpoMountData;
			
		gpoMountData = new Pline::MountData(*gpoData);
		*gpoMountData = *oVolumeConfig.gpoMountData;
	}
	
	// options
	gbAutomount = oVolumeConfig.gbAutomount;
	
	return this;
}

//==============================================================================

bool VolumeConfig::operator==(const VolumeConfig &oVolumeConfig) const
{
	// data
	if (!gpoData || !oVolumeConfig.gpoData)
		return false;
		
	if (*gpoData != *oVolumeConfig.gpoData)
		return false;
	
	// clone data
	if ((gpoCloneData && !oVolumeConfig.gpoCloneData)
		|| (!gpoCloneData && oVolumeConfig.gpoCloneData))
	{
		return false;
	}
	
	if (gpoCloneData && oVolumeConfig.gpoCloneData)
	{
		if (*gpoCloneData != *oVolumeConfig.gpoCloneData)
			return false;
	}
	
	// crypt data
	if ((gpoCryptData && !oVolumeConfig.gpoCryptData)
		|| (!gpoCryptData && oVolumeConfig.gpoCryptData))
	{
		return false;
	}
	
	if (gpoCryptData && oVolumeConfig.gpoCryptData)
	{
		if (*gpoCryptData != *oVolumeConfig.gpoCryptData)
			return false;
	}
	
	// mount data
	if ((gpoMountData && !oVolumeConfig.gpoMountData)
		|| (!gpoMountData && oVolumeConfig.gpoMountData))
	{
		return false;
	}
	
	if (gpoMountData && oVolumeConfig.gpoMountData)
	{
		if (*gpoMountData != *oVolumeConfig.gpoMountData)
			return false;
	}
	
	// options
	if (gbAutomount != oVolumeConfig.gbAutomount)
		return false;
		
	return true;
}

//==============================================================================

bool VolumeConfig::operator!=(const VolumeConfig &oVolumeConfig) const
{	
	return !((*this).operator==(oVolumeConfig));
}

//==============================================================================

bool VolumeConfig::cmpByPos(VolumeConfig* po1, VolumeConfig* po2)
{
	return po1->getData().getPos() < po2->getData().getPos();
}

//==============================================================================

const Glib::ustring VolumeConfig::getGroupNameFromLabel()
{
	size_t iPos = 0;
	Glib::ustring stGroupName = 
		Util::compose("DevConf/%s", gpoData->getLabel().c_str());
		
	while ((iPos = stGroupName.find(" ", iPos)) != std::string::npos) // escape spaces
	{
		stGroupName.replace(iPos, 1, "\\ ");
		iPos += 2;
	}
	
	return stGroupName;
}

//==============================================================================

bool VolumeConfig::loadConfig(  const Glib::ustring& stGroupName,
								Glib::KeyFile& oKeyFile)
{
	Glib::ustring st;
	
	gstGroupName.clear();
	
	try
	{   
		// data
		gpoData = new Pline::Data;
		
		// label
		size_t iPos = stGroupName.find("/");
		st = stGroupName;
		st.erase(0, iPos+1);
		
		while ((iPos = st.find("\\ ")) != std::string::npos) // un-escape spaces
			st.replace(iPos, 2, " ");
			
		gpoData->setLabel(st);
		
		gpoData->setPos(oKeyFile.get_integer(stGroupName, "Pos"));
		gpoData->setSrc(oKeyFile.get_string(stGroupName, "Src"));
		
		// clone data
		if (oKeyFile.has_key(stGroupName, "CloneDest"))
		{
			if (!(st = oKeyFile.get_string(stGroupName, "CloneDest")).empty())
			{
				gpoCloneData = new Pline::CloneData(*gpoData);

				gpoCloneData->setDest(st);
				gpoCloneData->setCanOverwrite(oKeyFile.get_boolean(stGroupName, "CloneOverwrite"));
			}
		}
		
		// crypt data
		if (oKeyFile.has_key(stGroupName, "Encrypted"))
		{
			if (oKeyFile.get_boolean(stGroupName, "Encrypted"))
			{
				gpoCryptData = new Pline::CryptData(*gpoData);
			
				gpoCryptData->setIsEncrypted(true);
				gpoCryptData->setKeyFile(oKeyFile.get_string(stGroupName, "Key"));
			}
		}
		
		// mount data
		gpoMountData = new Pline::MountData(*gpoData);
		
		gpoMountData->setDir(oKeyFile.get_string(stGroupName, "Dir"));
		gpoMountData->setDirOwner(oKeyFile.get_string(stGroupName, "DirOwner"));
		gpoMountData->setDirGroup(oKeyFile.get_string(stGroupName, "DirGroup"));
		gpoMountData->setDirPerm((unsigned long)oKeyFile.get_integer(stGroupName, "DirPerm"));
		gpoMountData->setDirSetPerm(oKeyFile.get_boolean(stGroupName, "DirSetPerm"));
		gpoMountData->setFSys(oKeyFile.get_string(stGroupName, "FSys"));
		gpoMountData->setReadOnly(oKeyFile.get_boolean(stGroupName, "ReadOnly"));
		gpoMountData->setMountFlags((unsigned long)oKeyFile.get_integer(stGroupName, "MountFlags"));
		
		if (oKeyFile.has_key(stGroupName, "EmptyTrash"))
			gpoMountData->setEmptyTrash(oKeyFile.get_boolean(stGroupName, "EmptyTrash"));
			
		if (oKeyFile.has_key(stGroupName, "EmptyTrashConfirm"))
			gpoMountData->setEmptyTrashConfirm(oKeyFile.get_boolean(stGroupName, "EmptyTrashConfirm"));
			
		// options
		if (oKeyFile.has_key(stGroupName, "Automount"))
			gbAutomount = oKeyFile.get_boolean(stGroupName, "Automount");
		
		gstGroupName = stGroupName;
		
		return true;
	}
	catch(...)
	{
		if (gpoData)
		{
			delete gpoData;
			gpoData = 0;
		}
		if (gpoCloneData)
		{
			delete gpoCloneData;
			gpoCloneData = 0;
		}
		if (gpoCryptData)
		{  
			delete gpoCryptData;
			gpoCryptData = 0;
		}
		if (gpoMountData)
		{
			delete gpoMountData;
			gpoMountData = 0;
		}
		
		return false;
	}
}

//==============================================================================

bool VolumeConfig::removeConfig(Glib::KeyFile& oKeyFile)
{
	try
	{
		if (!gstGroupName.empty())
			oKeyFile.remove_group(gstGroupName);
		else
			oKeyFile.remove_group(getGroupNameFromLabel());
		
		return true;
	}
	catch (...)
	{
		return false;
	}
}

//==============================================================================

bool VolumeConfig::saveConfig(Glib::KeyFile& oKeyFile)
{
	Glib::ustring stGroupNameInit;
	Glib::ustring stGroupName;
  
	if (!gpoData)
		return false;
		
	try
	{
		// label
		stGroupNameInit = gstGroupName;
		stGroupName = getGroupNameFromLabel();
					
		if (!stGroupNameInit.empty()) // volume was loaded
		{			
			if (stGroupNameInit != stGroupName) // volume label has changed
				oKeyFile.remove_group(stGroupNameInit);
		} // else this is a new volume
		
		if (gpoData)
		{
			oKeyFile.set_integer(stGroupName, "Pos", gpoData->getPos());
			oKeyFile.set_string(stGroupName, "Src", gpoData->getSrc());
		}
			
		if (gpoCloneData)
		{
			oKeyFile.set_string(stGroupName, "CloneDest", gpoCloneData->getDest());
			oKeyFile.set_boolean(stGroupName, "CloneOverwrite", gpoCloneData->getCanOverwrite());
		}
		else
		{
			oKeyFile.set_string(stGroupName, "CloneDest", "");
			oKeyFile.set_boolean(stGroupName, "CloneOverwrite", "");
		}
		
		if (gpoCryptData)
		{
			oKeyFile.set_boolean(stGroupName, "Encrypted", gpoCryptData->getIsEncrypted());
			oKeyFile.set_string(stGroupName, "Key", gpoCryptData->getKeyFile());	
		}
		else
		{
			oKeyFile.set_boolean(stGroupName, "Encrypted", false);
			oKeyFile.set_string(stGroupName, "Key", "");	
		}
			
		if (gpoMountData)
		{
			oKeyFile.set_string(stGroupName, "Dir", gpoMountData->getDir());
			oKeyFile.set_string(stGroupName, "DirOwner", gpoMountData->getDirOwner());
			oKeyFile.set_string(stGroupName, "DirGroup", gpoMountData->getDirGroup());
			oKeyFile.set_integer(stGroupName, "DirPerm", (int)gpoMountData->getDirPerm());
			oKeyFile.set_boolean(stGroupName, "DirSetPerm", gpoMountData->getDirSetPerm());
			oKeyFile.set_string(stGroupName, "FSys", gpoMountData->getFSys());
			oKeyFile.set_boolean(stGroupName, "ReadOnly", gpoMountData->getReadOnly());
			oKeyFile.set_integer(stGroupName, "MountFlags", (int)gpoMountData->getMountFlags());
			oKeyFile.set_boolean(stGroupName, "EmptyTrash", gpoMountData->isEmptyTrash());
			oKeyFile.set_boolean(stGroupName, "EmptyTrashConfirm", gpoMountData->isEmptyTrashConfirm());
		}
		
		// options
		oKeyFile.set_boolean(stGroupName, "Automount", gbAutomount);
		
		gstGroupName = stGroupName;
		
		return true;
	}
	catch(...)
	{
		return false;
	}
}

//==============================================================================
