/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * eMount
 * http://emount.sourceforge.net/
 * 
 * Copyright (C) Aqueio 2010-2011 <aqueio@yahoo.com>
 * 
 * eMount is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * eMount is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "eMountSys.h"
#include "QDebug.h"
#include "eMountSh.h"
#include "eMountWin.h"
#include "Icon.h"

#include <iostream>
#include <glibmm/i18n.h>
#include <algorithm>

// If QDEBUG is not defined read ui-file from string
#define UI_FILE "src/emount.ui"
#ifndef QDEBUG
#include "emount.ui.h"
#endif

//==============================================================================

OptionGroup::OptionGroup()
: Glib::OptionGroup("emount...", "description...", "help..."),
  gbList(false),
  gbVersion(false)
{
	const char* pszDiskLabel = _("<VOLUME_LABEL>");

	// FILES...
	/*
	Glib::OptionEntry entryFiles;
	entryFiles.set_long_name(G_OPTION_REMAINING);
	entryFiles.set_description(G_OPTION_REMAINING);
	entryFiles.set_arg_description(_("[FILE...]"));
	add_entry(entryFiles, gvecFile);
	*/

	// file_list
	/*
	Glib::OptionEntry entryFile;
	entryFiles.set_flags(Glib::OptionEntry::FLAG_HIDDEN|Glib::OptionEntry::FLAG_OPTIONAL_ARG); // Valid only in GUI mode
	entryFile.set_long_name("file");
	entryFile.set_short_name('f');
	entryFile.set_description(_("open a file (-f file1 -f file2 --file=file3)"));
	entryFile.set_arg_description(_("FILE"));
	add_entry(entryFile, gvecFile_list);
	*/
	
	Glib::OptionEntry entryEject;
	entryEject.set_long_name("eject");
	entryEject.set_short_name('e');
	entryEject.set_description(_("Eject volume"));
	entryEject.set_arg_description(pszDiskLabel);
	add_entry(entryEject, gstEject);
	
	Glib::OptionEntry entryList;
	entryList.set_long_name("list");
	entryList.set_short_name('l');
	entryList.set_description(_("Show volumes"));
	entryList.set_flags(Glib::OptionEntry::FLAG_NO_ARG);
	add_entry(entryList, gbList);
	
	Glib::OptionEntry entryMount;
	entryMount.set_long_name("mount");
	entryMount.set_short_name('m');
	entryMount.set_description(_("Mount volume"));
	entryMount.set_arg_description(pszDiskLabel);
	add_entry(entryMount, gstMount);
	
	Glib::OptionEntry entryPing;
	entryPing.set_long_name("ping");
	entryPing.set_short_name('p');
	entryPing.set_description(_("Return 0 if volume is mounted or in use"));
	entryPing.set_arg_description(pszDiskLabel);
	add_entry(entryPing, gstPing);
	
	Glib::OptionEntry entryStatus;
	entryStatus.set_long_name("status");
	entryStatus.set_short_name('s');
	entryStatus.set_description(_("Show volume status"));
	entryStatus.set_arg_description(pszDiskLabel);
	add_entry(entryStatus, gstStatus);
	
	Glib::OptionEntry entryVersion;
	entryVersion.set_long_name("version");
	entryVersion.set_short_name('v');
	entryVersion.set_description(_("Show program version"));
	entryVersion.set_flags(Glib::OptionEntry::FLAG_NO_ARG);
	add_entry(entryVersion, gbVersion);
}

//==============================================================================

OptionGroup::eMode OptionGroup::getMode(	Glib::ustring *pstDiskLabel,
											std::vector<Glib::ustring> *pvecFile) const
{
	if (pstDiskLabel)
		pstDiskLabel->clear();

	// copy files
	if (pvecFile)
		pvecFile->clear();
		
	for (std::vector<Glib::ustring>::size_type xi = 0; xi < gvecFile.size(); ++xi)
		pvecFile->push_back(gvecFile[xi]);
	
	for (std::vector<Glib::ustring>::size_type xi = 0; xi < gvecFile_list.size(); ++xi)
		pvecFile->push_back(gvecFile_list[xi]);
	
	// set mode
	if (gbList)
		return kModeList;
	else if (gbVersion)
		return kModeVersion;
	else if (!gstEject.empty())
	{
		*pstDiskLabel = gstEject;
			
		return kModeEject;
	}
	else if (!gstMount.empty())
	{
		*pstDiskLabel = gstMount;
			
		return kModeMount;
	}
	else if (!gstPing.empty())
	{
		*pstDiskLabel = gstPing;
			
		return kModePing;
	}
	else if (!gstStatus.empty())
	{
		*pstDiskLabel = gstStatus;
			
		return kModeStatus;
	}
	
	return kNoMode;
}

//==============================================================================

eMountSys::eMountSys()
: gpoKit(0),
  gpoOptions(0),
  geRunMode(kNoRun)
{
}

//==============================================================================

eMountSys::~eMountSys()
{
}

//==============================================================================

eMountSys& eMountSys::getInstance()
{
	static eMountSys oEmountSys;
	
	return oEmountSys;
}

//==============================================================================

int eMountSys::doGui(std::vector<Glib::ustring> *pvecFiles)
{	
	eMountWin *poMainWin = 0;
	
	try
	{
#ifdef QDEBUG
		gpoBuilder = Gtk::Builder::create_from_file(UI_FILE);
		
		debug_message(DEBUG_APP, "Reading UI file: %s", UI_FILE);
#else
		gpoBuilder = Gtk::Builder::create_from_string(UI_STRING);
#endif
	}
	catch (const Glib::FileError & ex)
	{
		std::cerr << ex.what() << std::endl;
		
		Gtk::MessageDialog oDialog(
			_("Failed loading GUI."),
			false,
			Gtk::MESSAGE_ERROR,
			Gtk::BUTTONS_OK,
			true
		);
		oDialog.run();
		
		return 1;
	}

	gpoBuilder->get_widget_derived("eMountWin", poMainWin);	
	
	if (poMainWin)
	{
		// set icon
		if (Icon::getInstance().drawSysIcon64())
		{
			poMainWin->set_icon(*Icon::getInstance().drawSysIcon64());
			poMainWin->set_default_icon(*Icon::getInstance().drawSysIcon64());
		}
		
		// taskbar
		/*
			Glib::RefPtr<Gtk::StatusIcon> oStatusIcon;
		
			if (Icon::getInstance().drawSysIcon16())
				doStatusIcon = Gtk::StatusIcon::create(*Icon::getInstance().drawSysIcon16());
		*/
				
		gpoKit->run(*poMainWin);

		delete poMainWin; // call distructor
		
		return 0;
	}

	return 1;
}

//==============================================================================

bool eMountSys::parse(int argc, char *argv[])
{
	Glib::OptionContext oContext("");	
	
	oContext.set_main_group(*gpoOptions);
	
	try
	{
		oContext.parse(argc, argv);
	}
	catch(const Glib::Error &ex)
	{
		std::cout << oContext.get_help(true);
		std::cerr << "Error loading options : " << ex.what() << std::endl;
		
		return false;
	}
	
	return true;
}

//==============================================================================

int eMountSys::run(int argc, char *argv[], Gtk::Main *poKit)
{
	int iRet;

	OptionGroup::eMode			eMode;
	OptionGroup					oOptions;
	Glib::ustring				stDiskLabel;
	std::vector<Glib::ustring>  vecFiles;
		
	if (geRunMode != kNoRun)
		return 1;
		
	gpoKit = poKit;
	gpoOptions = &oOptions;
	
	if (!parse(argc, argv))
		return 1;	

	eMode = gpoOptions->getMode(&stDiskLabel, &vecFiles);
	
	if (eMode == OptionGroup::kNoMode 
		|| eMode == OptionGroup::kModeFiles)
	{
		geRunMode = kRunGui;
		
		iRet = doGui(&vecFiles);
	}
	else
	{
		geRunMode = kRunSh;
		
		iRet = doShell(eMode, stDiskLabel);
	}
			
	return iRet;
}

//==============================================================================

int eMountSys::doShell( const OptionGroup::eMode eMode,
						Glib::ustring& stDiskLabel)
{
	return eMountSh::getInstance().run(eMode, stDiskLabel);
}

//==============================================================================
